# encoding: utf-8

from typing import List
from ..client.mongo_client import Mongo
from ..model.enums import CommonStatus


def get_spider_group_list(group_code: str, plat_codes: List[str], status: str = CommonStatus.ON):
    _query = {}
    if group_code:
        _query['taskType'] = group_code
    if plat_codes:
        _query['platCode'] = {"$in": plat_codes}

    _query['status'] = status

    return Mongo().list(collection="zcbot_stream_spider_group", query=_query)



# -*- coding: utf-8 -*-
from ..util import logger
from ..dao import batch_spider as batch_spider_dao

LOGGER = logger.get('批次爬虫')


def get_batch_spider(spider_id: str):
    row = batch_spider_dao.get_batch_spider(spider_id=spider_id)
    return row



# -*- coding: utf-8 -*-
from ..util import logger
from ..model.base import Platforms, Rule
from ..dao import meta as meta_dao

LOGGER = logger.get('基础')


# 获取支持网站平台
def get_platforms():
    rows = meta_dao.get_platforms()
    return [Platforms(**x) for x in rows]


# 获取链接分拣规则配置
def get_url_parse_rule(host: str = None):
    rows = meta_dao.get_url_parse_rule(host)
    return rows


def get_file_name_config(config_id: str = None):
    row = meta_dao.get_file_name_config(config_id)
    return row



# -*- coding: utf-8 -*-

from typing import List
from ..model.enums import CommonStatus
from ..model.entity import StreamSpiderModel
from ..dao import stream_spider_group


def get_spider_group_list(group_code: str, plat_codes: List[str], status: str = CommonStatus.ON) -> List[StreamSpiderModel]:
    rows = stream_spider_group.get_spider_group_list(group_code=group_code, plat_codes=plat_codes, status=status)
    rows = [StreamSpiderModel(**x) for x in rows]
    return rows





# -*- coding: utf-8 -*-
import redis
from ..util import cfg
from ..util.decator import singleton
from ..util.exceptions import NoConfigException


@singleton
class Redis(object):
    """
    Redis客户端简易封装（单例）
    """

    def __init__(self, redis_uri: str = None, redis_db: int = None, decode_responses=True):
        self.redis_uri = redis_uri or cfg.get('ZCBOT_CORE_REDIS_URL')
        self.redis_db = redis_db or cfg.get('ZCBOT_CORE_REDIS_DB')
        if not self.redis_uri:
            raise NoConfigException('redis uri not config!')
        if not self.redis_db:
            raise NoConfigException('redis database not config!')

        self.client = redis.Redis.from_url(url=self.redis_uri, db=self.redis_db, decode_responses=decode_responses)

    # 销毁关闭连接池
    def __del__(self):
        if self.client:
            self.client.close()



# -*- coding: utf-8 -*-
from typing import Optional
from ..client.mongo_client import Mongo
from ..model.entity import BatchSpider
from ..model.enums import CommonStatus


def _build_task_group_key(task_group, plat_code):
    return {'_id': f'{task_group}:{plat_code}:group'}


def _build_stream_spider_key(task_type, plat_code):
    return f'{task_type}:{plat_code}'


# 获取爬虫任务组配置
def get_spider_group(task_group, plat_code):
    return Mongo().get('zcbot_batch_spider_group', _build_task_group_key(task_group, plat_code))


# 获取portainer爬虫配置
def get_batch_spider(spider_id: str) -> Optional[BatchSpider]:
    row = Mongo().get('zcbot_batch_spider', {'spiderId': spider_id, 'status': CommonStatus.ON.name})
    if not row:
        return None
    return BatchSpider(**row)


# 获取支持网站平台
def get_platforms_by_spider_group(task_type: str):
    return Mongo().aggregate('zcbot_platforms', [
        {
            '$lookup': {
                'from': 'zcbot_batch_spider_group',
                'localField': '_id',
                'foreignField': 'plat_code',
                'as': 'spider'
            }
        },
        {
            '$match': {'spider.task_type': task_type.lower()}
        },
        {
            '$project': {'spider': 0}
        },
        {'$sort': {'sort': 1}}
    ])
